#!/usr/bin/env python

import sys
from openai import OpenAI
client = OpenAI()

completion = client.chat.completions.create(
  model="gpt-3.5-turbo",
  messages=[
    {"role": "system", "content": "You are a helpful assistant, skilled in explaining everything in simple terms even an 8 year old can understand."},
    {"role": "user", "content": sys.argv[1]}
  ]
)

print(completion.choices[0].message.content)



import sys

systemd_service = \
f"""
# save this into /etc/systemd/system/rxd.service
[Unit]
Description=rxd
After=network.target

[Service]
Type=simple
ExecStart=/usr/bin/screen -dmS rxd {sys.executable} -m rxd.daemon
TimeoutStopSec=5
Restart=always
RestartSec=3

[Install]
WantedBy=default.target
"""

if __name__ == "__main__":
    print(systemd_service)



import click
import json
from flask import Flask, request
from pprint import pprint

app = Flask(__name__)


@app.route('/', methods=['GET'])
def index():
    return "+OK"


@app.route('/receive_github_webhook', methods=['POST'])
def github_webhook_endpoint():
    print("github webhook recieved")
    pprint(request.json)
    return json.dumps(request.json)


@app.route('/receive_rxd', methods=['POST'])
def rxd_webhook_endpoint():
    from .messages import RXDRepoUpdatedEvent
    from .codex import Cipher
    cipher = Cipher(key="")
    print("rxd webhook recieved")
    pprint(request.json)
    data = request.json
    if data is None:
        return "+FAIL"

    message_enc = data['payload']
    message = cipher.decrypt(message_enc)
    event = RXDRepoUpdatedEvent.model_validate_json(message)

    print(event)

    return "+OK"

# ----------------
# Argument parsing
# ----------------


@click.group(name='responder')
def responder_cmd_group():
    """Web server to monitor for events"""
    pass


@responder_cmd_group.command()
def run():
    app.run(host='0.0.0.0', port=30475)


if __name__ == '__main__':
    responder_cmd_group()



import hashlib
import hmac
import typing as t
from cryptography.fernet import Fernet
from cryptography.fernet import Fernet


def verify_hmac_signature(secret: t.Union[str, bytes],
                          payload: t.Union[str, bytes],
                          signature: str):
    """Verify that the payload was sent from GitHub by validating SHA256.

    Raise and return 403 if not authorized.

    Args:
        payload: original request body to verify (request.body())
        secret: GitHub app webhook token (WEBHOOK_SECRET)
        signature: header received from GitHub (x-hub-signature-256)
    """
    if not signature:
        return False

    expected_signature = generate_hmac_signature(secret, payload)

    # signature generated from body did not match expected_signature
    if not hmac.compare_digest(expected_signature, signature):
        return False

    return True


def generate_hmac_signature(secret: t.Union[str, bytes],
                            payload: t.Union[str, bytes]):
    """
    Generate a signature given secret and payload
    """
    payload_bytes = payload.encode('utf-8') \
        if isinstance(payload, str) \
        else payload

    secret_bytes = secret.encode('utf-8') \
        if isinstance(secret, str) \
        else secret

    hash = hmac.new(secret_bytes,
                    msg=payload_bytes,
                    digestmod=hashlib.sha256)
    return "sha256=" + hash.hexdigest()


class Cipher:

    def __init__(self, key=None):
        self.key = key

    def generate_key(self) -> bytes:
        self.key = Fernet.generate_key()
        return self.key

    def encrypt(self, message: str) -> bytes:
        if self.key is None:
            raise Exception("Key has not been set")

        f = Fernet(self.key)
        return f.encrypt(message.encode('utf-8'))

    def decrypt(self, encrypted_message: bytes) -> str:
        if self.key is None:
            raise Exception("Key has not been set")

        f = Fernet(self.key)
        return f.decrypt(encrypted_message).decode('utf-8')

def test_hmac_verification():
    secret = "hello world"
    payload = "This is a test payload"
    signature = generate_hmac_signature(secret, payload)
    is_verified = verify_hmac_signature(secret, payload, signature)
    assert is_verified, "generated signature does not match expected signature"

def test_Cipher():
    message = "This is a test payload"
    Cipher = Cipher()
    Cipher.generate_key()
    encrypted_message = Cipher.encrypt(message)
    decrypted_message = Cipher.decrypt(encrypted_message)
    assert message == decrypted_message

    # generate a new key and make sure we can not decrypt
    Cipher.generate_key()
    decrypted_message = Cipher.decrypt(encrypted_message)
    assert message == decrypted_message



import sys
import os
import subprocess as sp
import json
import typing as t
import textwrap
import getpass
from pathlib import Path

HOME = '~/.rxd'
HOME = Path(HOME).expanduser().resolve()
APP_DIR = Path(HOME, "apps")
WORKSPACE_DIR = Path('~/workspace').expanduser().resolve()


class Chdir:

    def __init__(self, directory):
        self.directory = directory
        self.oldcwd = None

    def __enter__(self):
        print("changing directory to %s" % self.directory)
        self.oldcwd = Path(os.curdir).resolve()
        os.chdir(self.directory)

    def __exit__(self, type, value, traceback):

        if self.oldcwd is not None:
            print("changing directory back to %s" % self.oldcwd)
            os.chdir(self.oldcwd)
            self.oldcwd = None


class AppManager:
    def __init__(self):
        pass

    def list(self):
        return [Application.load(app_file.split(".json")[0])
                for app_file in os.listdir(APP_DIR)]

    def add(self, name, repo):
        app = Application(name, repo)
        app.save()

    def delete(self, name):
        return Application.load(name).delete()

    def fetch(self, name):
        app = Application.load(name)
        app.fetch()


class Application:
    def __init__(self, name,
                 repo=None) -> None:
        self.name = name
        self.repo = repo

    @property
    def repo_name(self) -> t.Union[str, None]:
        if self.repo:
            return str(Path(self.repo).stem)
        return None

    @property
    def repo_path(self) -> t.Union[Path, None]:
        if self.repo and self.repo_name:
            return self.workspace_container_path\
                       .joinpath(self.repo_name)\
                       .resolve()
        return None

    @property
    def metadata_path(self) -> Path:
        return Path(HOME, "apps", "%s.json" % self.name).resolve()

    @property
    def workspace_container_path(self) -> Path:
        return Path(WORKSPACE_DIR, self.name).resolve()

    @property
    def systemd_services_definition_path(self) -> Path:
        return self.workspace_container_path.joinpath("app.service").resolve()

    @property
    def systemd_service_name(self):
        return f"rxd-app-{self.name}.service"

    def save(self):
        self.setup_metadata()
        path = self.metadata_path
        with open(path, "w") as fh:
            json.dump({
                "name": self.name,
                "repo": self.repo
            }, fh)

    def delete(self):
        os.remove(self.metadata_path)

    def setup_metadata(self):
        if not self.metadata_path.parent.exists():
            os.makedirs(self.metadata_path.parent)

    def setup_workspace(self):
        if not self.workspace_container_path.exists():
            os.makedirs(self.workspace_container_path)

    def exists(self):
        return self.metadata_path.exists()

    @classmethod
    def load(cls, name):
        metadata_path = Path(HOME,
                             "apps",
                             "%s.json" % name).resolve()

        if metadata_path.exists():
            with open(metadata_path, "r") as fh:
                data = json.load(fh)
                return Application(name=data['name'],
                                   repo=data['repo'])
        else:
            return Application(name=name)

    def fetch(self):
        self.setup_workspace()

        # if no repo
        if self.workspace_container_path.exists() \
                and self.repo_path \
                and not self.repo_path.exists():
            with Chdir(self.workspace_container_path):
                sp.check_call("git clone --depth 1 %s" % self.repo,
                              shell=True)

        # if repo exists
        elif self.workspace_container_path.exists()\
                and self.repo_path \
                and self.repo_path.exists():
            with Chdir(self.repo_path):
                sp.check_call("git pull origin main",
                              shell=True)

    def build(self):
        if repo_path := self.repo_path:
            with Chdir(repo_path):
                sp.check_call("/usr/bin/bash .deploy/build", shell=True)

    def run(self):
        if repo_path := self.repo_path:
            with Chdir(repo_path):
                sp.check_call("/usr/bin/bash .deploy/run", shell=True)

    def daemonize(self):
        screen_name = self.name
        python_path = sys.executable
        user = getpass.getuser()
        systemd_service_file = \
            f"""
            [Unit]
            Description=rxd
            After=network.target
            StartLimitIntervalSec=30
            StartLimitBurst=5

            [Service]
            Type=simple
            User={user}
            ExecStart={python_path} -m rxd.cmd app run {self.name}
            Restart=always
            RestartSec=5

            [Install]
            WantedBy=default.target
            """
        systemd_service_file = textwrap.dedent(systemd_service_file)
        systemd_service_install_path = Path(
            f'/etc/systemd/system/{self.systemd_service_name}')
        print(f'Writing {self.systemd_services_definition_path}')
        with open(self.systemd_services_definition_path, "w") as fh:
            fh.write(systemd_service_file)

        if Path('/etc/systemd/system').exists():
            if not systemd_service_install_path.exists():

                print("Moving %s -> %s" %
                      (self.systemd_services_definition_path,
                       systemd_service_install_path))
                print("We need sudo permissions to do so")
                sp.check_call(
                    ["sudo", "mv",
                     self.systemd_services_definition_path,
                     systemd_service_install_path])
                sp.check_call(["sudo", "systemctl",
                               "daemon-reload"])
        else:
            print("Systemd does not exist")

    def status(self, output=False):
        if not Path('/etc/systemd/system').exists():
            return None

        lines = sp.check_output(["systemctl",
                                 "show",
                                 "--no-pager",
                                 self.systemd_service_name])
        lines = lines.decode("utf-8").split("\n")
        status_data = {}
        for line in lines:
            kv = line.strip().split("=", 1)
            if len(kv) == 2:
                key, value = kv
                status_data[key] = value

        status = dict(
            is_running=status_data.get('ActiveState') == 'active',
            pid=int(status_data.get('MainPID', -1)),
            user=status_data.get('User'),
            memory_mb=int(status_data.get('MemoryCurrent', -1))/(1024*1024),
            state=status_data.get('UnitFileState')
        )
        if output:
            print(json.dumps(status, indent=4))
        return status

    def start(self):
        sp.check_output(["sudo",
                         "systemctl",
                         "start",
                         self.systemd_service_name])

    def stop(self):
        sp.check_output(["sudo",
                         "systemctl",
                         "stop",
                         self.systemd_service_name])

    def restart(self):
        sp.check_output(["sudo",
                         "systemctl",
                         "restart",
                         self.systemd_service_name])
        pass

    def enable(self):
        sp.check_output(["sudo",
                         "systemctl",
                         "enable",
                         self.systemd_service_name])

    def disable(self):
        sp.check_output(["sudo",
                         "systemctl",
                         "disable",
                         self.systemd_service_name])

    def logs(self, follow=False):
        if follow:
            sp.check_output(["journalctl",
                             "-f",
                             "-u",
                             self.systemd_service_name])
        else:
            sp.check_output(["journalctl",
                             "-u",
                             self.systemd_service_name])

    def __repr__(self):
        return "Application(name=%s, repo=%s)" % (self.name, self.repo)
